"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var systems_1 = require("./systems");
var entity_1 = require("./entity");
var entity_manager_1 = require("./entity-manager");
function start(systems) {
    var cleanups = [];
    var events = [];
    var entityManager = new entity_manager_1.EntityManager();
    var running = false;
    function createWorld(event) {
        return {
            event: event,
            query: entityManager.query,
            queryOne: entityManager.queryOne,
            add: add,
            remove: entityManager.scheduleRemove,
            emit: emit
        };
    }
    function add(components) {
        if (components === void 0) { components = []; }
        return components.reduce(function (e, c) { return e.add(c); }, new entity_1.Entity(entityManager.scheduleUpdate));
    }
    function emit(event) {
        if (running) {
            events.push(event);
        }
        else {
            setTimeout(function () { return runSystems(event); });
        }
    }
    function runSystems(event) {
        running = true;
        var world = createWorld(event);
        for (var _i = 0, systems_2 = systems; _i < systems_2.length; _i++) {
            var system = systems_2[_i];
            runSystem(system, world);
            entityManager.processUpdates();
        }
        running = false;
    }
    function runSystem(system, world) {
        try {
            var result = system(world);
            if (typeof result === 'function') {
                cleanups.push(result);
            }
        }
        catch (e) {
            console.error(e);
        }
    }
    setTimeout(function () { return runSystems(new systems_1.InitEvent()); });
    return function () {
        cleanups.forEach(function (cleanup) { return cleanup(); });
    };
}
exports.start = start;
//# sourceMappingURL=engine.js.map