"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function hasAll(components) {
    return function (entity) { return components.every(function (component) { return entity.has(component); }); };
}
exports.hasAll = hasAll;
var Query = /** @class */ (function () {
    function Query(selector, entities) {
        var _this = this;
        this.selector = selector;
        this.entities = [];
        this.indices = new Map();
        entities
            .filter(this.selector)
            .forEach(function (entity) { return _this.onChange(entity); });
    }
    Query.prototype.onChange = function (entity) {
        if (this.selector(entity)) {
            if (this.indices.get(entity) == null) {
                this.indices.set(entity, this.entities.length);
                this.entities.push(entity);
            }
        }
        else {
            this.onRemove(entity);
        }
    };
    Query.prototype.onRemove = function (entity) {
        var index = this.indices.get(entity);
        if (index != null) {
            var last = this.entities.pop();
            if (last !== entity) {
                this.entities[index] = last;
                this.indices.set(last, index);
            }
            this.indices.delete(entity);
        }
    };
    return Query;
}());
exports.Query = Query;
//# sourceMappingURL=query.js.map