import { InitEvent } from './systems';
import { Entity } from './entity';
import { EntityManager } from './entity-manager';
export function start(systems) {
    var cleanups = [];
    var events = [];
    var entityManager = new EntityManager();
    var running = false;
    function createWorld(event) {
        return {
            event: event,
            query: entityManager.query,
            queryOne: entityManager.queryOne,
            add: add,
            remove: entityManager.scheduleRemove,
            emit: emit
        };
    }
    function add(components) {
        if (components === void 0) { components = []; }
        return components.reduce(function (e, c) { return e.add(c); }, new Entity(entityManager.scheduleUpdate));
    }
    function emit(event) {
        if (running) {
            events.push(event);
        }
        else {
            setTimeout(function () { return runSystems(event); });
        }
    }
    function runSystems(event) {
        running = true;
        var world = createWorld(event);
        for (var _i = 0, systems_1 = systems; _i < systems_1.length; _i++) {
            var system = systems_1[_i];
            runSystem(system, world);
            entityManager.processUpdates();
        }
        running = false;
    }
    function runSystem(system, world) {
        try {
            var result = system(world);
            if (typeof result === 'function') {
                cleanups.push(result);
            }
        }
        catch (e) {
            console.error(e);
        }
    }
    setTimeout(function () { return runSystems(new InitEvent()); });
    return function () {
        cleanups.forEach(function (cleanup) { return cleanup(); });
    };
}
//# sourceMappingURL=engine.js.map