import { createSystem } from './createSystem';
import { InitEvent } from './init-event';
var UpdateTick = /** @class */ (function () {
    function UpdateTick(timestamp, deltaTime) {
        this.timestamp = timestamp;
        this.deltaTime = deltaTime;
    }
    return UpdateTick;
}());
export { UpdateTick };
var RenderTick = /** @class */ (function () {
    function RenderTick() {
    }
    return RenderTick;
}());
export { RenderTick };
function scheduleRaf(callback) {
    var id = requestAnimationFrame(callback);
    return function () { return cancelAnimationFrame(id); };
}
function scheduleTimeout(callback) {
    var id = setTimeout(callback, 16);
    return function () { return clearTimeout(id); };
}
var scheduleDefault = typeof requestAnimationFrame === 'function' ? scheduleRaf : scheduleTimeout;
export function gameClock(ticksPerSecond, schedule) {
    if (ticksPerSecond === void 0) { ticksPerSecond = 60; }
    if (schedule === void 0) { schedule = scheduleDefault; }
    var deltaMs = 1000 / ticksPerSecond;
    return createSystem(InitEvent, function (world) {
        var lastTime = Date.now();
        var stop = function () { };
        update();
        function update() {
            var now = Date.now();
            while (lastTime < now) {
                world.emit(new UpdateTick(now, 1 / ticksPerSecond));
                lastTime += deltaMs;
            }
            world.emit(new RenderTick());
            stop = schedule(update);
        }
        return function () { return stop(); };
    });
}
//# sourceMappingURL=game-clock.js.map